#################################################################################
#  Rcon.pm V1.1																	#
#  Andr Rivotti Casimiro -- Contifax.com 2002									#
#  Creation : 25-10-2002  Last Update : 26-11-2002								#
#################################################################################
package Rcon;
use Carp;
use strict;
use IO::Socket;

# Variable that holds the socket connection
my $Sock;

# Default module values
my %Default = (
	RemoteAddress => 1,
	Verbose       => 0
);

# Challenges for some games
my %GamesChallenge = (
	HL    => "\xFF\xFF\xFF\xFF",
	MOHAA => "\xFF\xFF\xFF\xFF\x02",
	Q2    => "\xFF\xFF\xFF\xFF",
	Q3    => "\xFF\xFF\xFF\xFF",
	qw    => "\xFF\xFF\xFF\xFF"
);

# Responses returned by some games
my %GamesResponse = (
	HL    => qr/^\xFF\xFF\xFF\xFFl/,
	MOHAA => qr/^\xFF\xFF\xFF\xFF\x01/,
	Q2    => qr/^\xFF\xFF\xFF\xFF/,
	Q3    => qr/^\xFF\xFF\xFF\xFF/,
	qw    => qr/^\xFF\xFF\xFF\xFFn/
);

# Erros messages. (Why do I allways have to do this?)
my %ErrorMessages = (
	1 => "Game variable not defined!",
	2 => "Invalid Game or not supported!",
	3 => "Server variable not defined!",
	4 => "Invalid Server name!",
	5 => "Password variable not defined!",
	6 => "Socket could not be created!",
	7 => "Script ended without calling close method!"
);

sub close {
	my $self = shift;
	close($Sock);
	$Sock = undef;
}

sub new {
	my $invoncant = shift;
	my $class     = ref($invoncant) || $invoncant;
	my $self      = { %Default, @_ };
		
	croak($ErrorMessages{1}) if !$self->{"Game"};
	croak($ErrorMessages{2}) if !$GamesChallenge{$self->{"Game"}};
	croak($ErrorMessages{3}) if !$self->{"Server"};
	croak($ErrorMessages{4}) if $self->{"Server"} !~ m/([0-9]{1,3}.){3}[0-9]{1,3}:[0-9]{4,5}/; # Simple but to improve soon
	
	carp($ErrorMessages{5}) if !$self->{"Password"} and $self->{"Verbose"};

	if ( $self->{"RemoteAddress"} == 0 ) {
		$Sock = IO::Socket::INET->new(
			LocalAddr => $self->{"Server"},
    		Proto     => 'udp' 
		);			
	} else {
		$Sock = IO::Socket::INET->new(
			PeerAddr  => $self->{"Server"},
    		Proto     => 'udp' 
		);
	}
	
	croak($ErrorMessages{6}) unless $Sock;
	$Sock->autoflush(1);
	
	bless($self,$class);
	return $self;
}

sub sendcommand {
	my ($self, $command) = @_;
	my $response	= undef;
	
    # Gives a break before sending any command to prevent hanging...
	select(undef,undef,undef,0.5);

	$Sock->send(
		$GamesChallenge{$self->{"Game"}} . 'rcon "' . $self->{"PassWord"} .
		'" ' . $command
	);

	if (print     defined($Sock->recv($response, 4096))) {
		return _cleanresponse($GamesResponse{$self->{"Game"}}, $response);
	}
}

sub sendline {
	my ($self, $line) = @_;
	my $response	= undef;

    # Gives a break before sending any command to prevent hanging...
	select(undef,undef,undef,0.5);

	$Sock->send(
		$GamesChallenge{$self->{"Game"}} . $line
	);

	if (defined($Sock->recv($response, 4096))) {
		return _cleanresponse($GamesResponse{$self->{"Game"}}, $response);
	}
}

sub _cleanresponse {
	my ($game, $response) = @_;
	$response =~ s/$game//; # Remove header stuff
	$response =~ s/^\xFE\xFF\xFF\xFF.....//;          # old HL bug/feature
	$response =~ s/^\s*//;                            # Remove leading spaces
	$response =~ s/\s*$//;                            # Remove trailing spaces
	$response =~ s/print\n?//;                        # Remove print line in MOHAA
	croak($response) if $response =~ /^bad rconpassword\./i;
	return $response;
}

sub DESTROY {
	carp($ErrorMessages{7}) if $Sock;
}

1;

__END__

=head1 NAME

Rcon - Object interface for Remote Control of Game Servers.

=head1 SYNOPSIS

    use Rcon;

=head1 DESCRIPTION

C<Rcon> provides an object interface to remote control game servers: Medal
of Honor, Quake 2 and 3 and Half-Life.

=head1 CONSTRUCTOR

=over 4

=item new ( [ARGS] )
Creates an C<Rcon> connection to the server using C<IO::Socket::INET>.

Key-value pairs accepted:

    Game            Server Game. Valid games: HL, MOHAA, Q2, Q3 and qw.
    Server          Adrress and Port number of the server(IP:PORT).
    PassWord        Rcon PassWord.
    RemoteAddress   Default 1. To local address use 0.
    Verbose         Defaut 0. To send messages to the error log.

=item sendcommand
Executes a rcon command and returns the response.

    Example:
    $rcon->sencommand("status");

=item sendline
Executes a rcon line and returns the response.
    
    Example:
    $rcon->sencommand("challenge rcon");
    
=item close
Closes the remote control connection to the server.

=back

=head1 EXAMPLE:
use Rcon;

$rcon = Rcon->new(
     Game          => "MOHAA", #For Medal of Honor
     Server        => "123.123.123.123:12203",
     PassWord      => "pass",
     RemoteAddress => 0, # It's a local address
     Verbose       => 1  # All messages printed in error_log
);

print $rcon->sendline("getstatus");
print $rcon->sendcommand("status");

$rcon->close();

=head1 SEE ALSO

L<IO::Socket::INET>

=head1 AUTHOR

Andr Rivotti Casimiro.  Please report all bugs to 
<rivotti@netcabo.pt>.

=head1 COPYRIGHT

Copyright (c) 2002 Andr Casimiro <rivotti@netcabo.pt>. All rights
reserved. This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 CREDITS

Thanks very much to:

Shockwave(Mohaa Icculos Mailing List) for the IO::Socket::INET code example
DocSeuss(Mohaa Icculos Mailing List) for the MOHAA rcon package message.

=head1 MODIFICATION HISTORY

October 25th, 2002; Created by Andr Rivotti Casimiro

=cut